import socket
import numpy as np
import struct
import common


class sMessage:
    def __init__(self):
    	self.typ = 0
	self.length = 0
	self.data = ''

class FTCommands:
    def __init__(self):
        """ constructor, setting initial variables """
	self.dither_freq = 30
	self.dither_amp = 10.5
	self.opdamp = 13.2
	self.dither_smoothfrac = 0.05
        self.dither_delay = 0.0

 	self.search_threshold=np.array([1000.,1000.,1000.,1000.,1000.,1000.])
	self.paddleStep = 1.0
	self.gain=np.array([1,1,1,-.03,1,1])
	self.searchSNRThr=np.array([1000.,1000.,1000.,1000.,1000.,1000.])
	self.semiLockSNRThr=np.array([1000.,1000.,1000.,1000.,1000.,1000.])
	self.lockSNRThr=np.array([1000.,1000.,1000.,1000.,1000.,1000.])

	self.ampThr=np.array([1000.,1000.,1000.,1000.,1000.,1000.])
	
	self.toggledBC = np.array([999, 999, 999, 999, 999, 999])
	self.champStatus = np.array([999, 999, 999, 999, 999, 999])
	self.BCRefOffset = np.array([1000.,10000.,0.,0.,0.,0.])
	self.BCFTOffset = np.array([0.,0.,0.,0.,0.,0.])
	self.delayOffset = np.array([0.,0.,0.,0.,0.,0.])
	
	self.searchMode = np.array([1,1,1,1,1,1])
	self.searchSpeed = np.array([30., 30., 30., 30., 30., 30.])
        self.searchStep0 = np.array([-10.0,-10.0,-10.0,-10.0,-10.0,-10.0])
	
	self.champState = np.array([0, 0, 0, 0, 0, 0])
	self.champSetState = np.array([999, 999, 999, 999, 999, 999])


	self.smessage = sMessage()

    def initSocket(self):
        self.sock = None
	#common.port = 60000
        for res in socket.getaddrinfo(common.host, common.port, socket.AF_UNSPEC, socket.SOCK_STREAM):
            af, socktype, proto, canonname, sa = res
            try:
                self.sock = socket.socket(af, socktype, proto)
		self.sock.settimeout(common.timeout)
            except socket.error, msg:
                self.sock = None
                continue
            try:
                self.sock.connect(sa)
                self.fd = self.sock.fileno()
            except socket.error, msg:
                self.sock.close()
                self.sock = None
                continue
            break
        if self.sock is None:
            print 'could not open socket'
            

    def updatePZTStructure(self):
        self.initSocket()
        c_struct = struct.pack(
            '5f',
	    self.dither_freq,
	    self.dither_amp,
	    self.opdamp,
	    self.dither_smoothfrac,
            self.dither_delay,            )       
        strLen = len(c_struct)
        print strLen
        self.sock.send('FT')
        print "sent FT"
        self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
        self.sock.send(c_struct)
        self.sock.close()

    def updateServoStructure(self):
        self.initSocket()
        c_struct = struct.pack(
            '6f6f6f6f',
	    self.gain[0], self.gain[1], self.gain[2],
	    self.gain[3], self.gain[4], self.gain[5], 
	    self.searchSNRThr[0], self.searchSNRThr[1], self.searchSNRThr[2],
	    self.searchSNRThr[3], self.searchSNRThr[4], self.searchSNRThr[5], 
	    self.semiLockSNRThr[0], self.semiLockSNRThr[1], self.semiLockSNRThr[2],
	    self.semiLockSNRThr[3], self.semiLockSNRThr[4], self.semiLockSNRThr[5], 
	    self.lockSNRThr[0], self.lockSNRThr[1], self.lockSNRThr[2],
	    self.lockSNRThr[3], self.lockSNRThr[4], self.lockSNRThr[5],
	    )       
        strLen = len(c_struct)
        print strLen
        self.sock.send('CP')
        print "sent CP"
        self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
        self.sock.send(c_struct)
        self.sock.close()
	
    def paddleFringe(self, bc, paddleOffset):
	print paddleOffset
        self.initSocket()
        c_struct = struct.pack(
            'id',
	    bc, 
	    paddleOffset,
	    )       
        strLen = len(c_struct)
        print strLen
        self.sock.send('PA')
        print "sent PA"
        self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
        self.sock.send(c_struct)
        self.sock.close()
	

    def setSearchParameters(self):
        self.initSocket()
        c_struct = struct.pack(
            '6i6f6d',
	    self.searchMode[0], self.searchMode[1], self.searchMode[2],
	    self.searchMode[3], self.searchMode[4], self.searchMode[5], 
	    self.searchSpeed[0], self.searchSpeed[1], self.searchSpeed[2],
	    self.searchSpeed[3], self.searchSpeed[4], self.searchSpeed[5],
	    self.searchStep0[0], self.searchStep0[1], self.searchStep0[2],
	    self.searchStep0[3], self.searchStep0[4], self.searchStep0[5], 
	    )       
        strLen = len(c_struct)
        print strLen
        self.sock.send('SS')
        print "sent SS"
        self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
        self.sock.send(c_struct)
        self.sock.close()
        print self.searchMode[0], self.searchMode[1], self.searchMode[2], self.searchMode[3], self.searchMode[4], self.searchMode[5], self.searchSpeed[0], self.searchSpeed[1], self.searchSpeed[2], self.searchSpeed[3], self.searchSpeed[4], self.searchSpeed[5], self.searchStep0[0], self.searchStep0[1], self.searchStep0[2], self.searchStep0[3], self.searchStep0[4], self.searchStep0[5]
	
    def setAmpThr(self):
        self.initSocket()
	c_struct = struct.pack(
	    '6f',
	    self.searchMode[0], self.searchMode[1], self.searchMode[2],
	    self.searchMode[3], self.searchMode[4], self.searchMode[5], 
	    self.searchSpeed[0], self.searchSpeed[1], self.searchSpeed[2],
	    self.searchSpeed[3], self.searchSpeed[4], self.searchSpeed[5],
	    self.searchStep0[0], self.searchStep0[1], self.searchStep0[2],
	    self.searchStep0[3], self.searchStep0[4], self.searchStep0[5], 
	    )       
	strLen = len(c_struct)
	print strLen
	self.sock.send('AT')
	print "sent AT"
	self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
	self.sock.send(c_struct)
	self.sock.close()
	
    def calcAmpThr(self):
	length =  6 * 4		#6 floats
        self.smessage.length = length
        self.initSocket()
	self.sock.settimeout(30)
	self.sock.send('CT')
	self.sock.send(struct.pack('L', self.smessage.length))
	length = 0
	data = ''
	while length < self.smessage.length:
	    self.smessage.data = self.sock.recv(self.smessage.length-length)
	    length += len(self.smessage.data)
	    data += self.smessage.data
	self.sock.close()
	self.ampThr = np.fromstring(data, np.float32, 6)
	
    def champCommand(self):
	print "self.champSetState = ", self.champSetState
	self.initSocket()
	if self.sock != None:
	    c_struct = struct.pack(
		'6i',
		self.champSetState[0], self.champSetState[1], self.champSetState[2],
		self.champSetState[3], self.champSetState[4], self.champSetState[5],
		)       
	    strLen = len(c_struct)
	    self.sock.send('CO')
	    print "sent CO"
	    self.sock.send(struct.pack('L',strLen))
	    self.sock.send(c_struct)
	    self.sock.close()
	
    def getChampStatus(self):
	length =  6*8 + 6*8 + 6*4		#6 doubles 6 doubles 4 int
        self.smessage.length = length
        self.initSocket()
	if self.sock != None:
	    self.sock.settimeout(30)
	    self.sock.send('CS')
	    self.sock.send(struct.pack('L', self.smessage.length))
	    length = 0
	    data = ''
	    while length < self.smessage.length:
	        self.smessage.data = self.sock.recv(self.smessage.length-length)
	        length += len(self.smessage.data)
	        data += self.smessage.data
	    self.sock.close()
	    self.BCRefOffset = np.fromstring(data[0:48], np.float64, 6)
	    self.BCFTOffset = np.fromstring(data[48:96], np.float64, 6)
	    self.champState = np.fromstring(data[96:120], np.int, 6)
	
    def setBCRefOffset(self):
        self.initSocket()
	c_struct = struct.pack(
	    '6i6d',
	    self.toggledBC[0], self.toggledBC[1], self.toggledBC[2],
	    self.toggledBC[3], self.toggledBC[4], self.toggledBC[5], 
	    self.BCRefOffset[0], self.BCRefOffset[1], self.BCRefOffset[2],
	    self.BCRefOffset[3], self.BCRefOffset[4], self.BCRefOffset[5], 
	    )       
	strLen = len(c_struct)
	print strLen
	self.sock.send('SR')
	print "sent SR"
	self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
	self.sock.send(c_struct)
	self.sock.close()
	
    def initBCRefOffset(self):
	length =  6*8		#6 doubles 6 doubles 4 int
        self.smessage.length = length
        self.initSocket()
	self.sock.settimeout(30)
	self.sock.send('IR')
	self.sock.send(struct.pack('L', self.smessage.length))
	length = 0
	data = ''
	while length < self.smessage.length:
	    self.smessage.data = self.sock.recv(self.smessage.length-length)
	    length += len(self.smessage.data)
	    data += self.smessage.data
	self.sock.close()
	self.BCRefOffsets = np.fromstring(data, np.float64, 6)
	
    def setDelayOffset(self):
        self.initSocket()
	c_struct = struct.pack(
	    '6i6d',
	    self.toggledBC[0], self.toggledBC[1], self.toggledBC[2],
	    self.toggledBC[3], self.toggledBC[4], self.toggledBC[5], 
	    self.delayOffset[0], self.delayOffset[1], self.delayOffset[2],
	    self.delayOffset[3], self.delayOffset[4], self.delayOffset[5], 
	    )       
	strLen = len(c_struct)
	print strLen
	self.sock.send('DO')
	print "sent DO"
	self.sock.send(struct.pack('L',strLen))
	print "sent ", strLen
	self.sock.send(c_struct)
	self.sock.close()
	
#    def getFringeData(self):
#fringeArr = np.zeros((nbpxFringe, nbFringe), dtype=np.float32)
#phaseArr = np.zeros((nbpxFringe, nbFringe), dtype=np.float32)
#jitterArr = np.zeros((nbpxFringe, nbFringe), dtype=np.float32)
#        for ndx in range(common.nBL):
#            for npxFringe in range(common.nbpxFringe):
#                common.fringeArr[npxFringe][ndx] = npxFringe*1.
#
#        common.dataFluxReady = 1
#
#        print "RUN getFringeData"

