#! /usr/bin/env python
'''Application main window '''

import gobject
import gtk
import champDisplay
import champHelp
import champControls
import common
import FTCommands
from matplotlib.backends.backend_gtk import NavigationToolbar2GTK as NavigationToolbar
import numpy as np

#Allow any thread to do GUI stuff
gtk.gdk.threads_init()

ui_info = \
'''<ui>
  <menubar name='MenuBar'>
    <menu action='Cop'>
      <menuitem action='Set server config'/>
      <menuitem action='Set BC config'/>
      <menuitem action='Set PZT parameters'/>
      <menuitem action='Set FT parameters'/>
      <menuitem action='Set/Get offsets'/>
      <menuitem action='Quit'/>
    </menu>
    <menu action='HelpMenu'>
      <menuitem action='Help'/>
    </menu>
  </menubar>
</ui>'''

def create_arrow_button(arrow_type, shadow_type):
    button = gtk.Button();
    arrow = gtk.Arrow(arrow_type, shadow_type);
    button.add(arrow)
    button.show()
    arrow.show()
    return button

class ApplicationMainWindow(gtk.Window):
    def __init__(self, parent=None):
        self.anim = champDisplay.AnimatedImage()
        self.ft = FTCommands.FTCommands()
        self.tagDisplayOn = -1
        self.tagFTLoop = -1
        searchIcon = 'Icons/system-search.png'
        offIcon = 'Icons/gtk-close.png'
        semiLockIcon = 'Icons/stock_lock.png'
        lockIcon = 'Icons/stock_lock-ok.png'
        self.stateIcons = {0:offIcon, 1:searchIcon, 2:semiLockIcon, 3:lockIcon}

        # Create the toplevel window
        gtk.Window.__init__(self)
        try:
            self.set_screen(parent.get_screen())
        except AttributeError:
            self.connect('destroy', lambda *w: gtk.main_quit())
        
        self.set_title("CHAMP OBSERVATION PANEL")
        self.set_default_size(1200, 800)
        
        merge = gtk.UIManager()
        self.set_data("ui-manager", merge)
        merge.insert_action_group(self.__create_action_group(), 0)
        self.add_accel_group(merge.get_accel_group())

        try:
            mergeid = merge.add_ui_from_string(ui_info)
        except gobject.GError, msg:
            print "building menus failed: %s" % msg
        bar = merge.get_widget("/MenuBar")
        bar.show()
        
        #Create large vertical box
        bigVBox = gtk.VBox(False, 0)
        bigVBox.pack_start(bar, False, False)
        vpaned = gtk.VPaned()
        bigVBox.pack_start(vpaned, True, True)
        vpaned.set_border_width(5)

        #horizontal box for the fringes
        self.searchStep0Entry = {}
        self.searchSpeedEntry = {}
        self.paddleStepEntry = {}
        self.ampThrEntry = {}
        self.tel1Entry = {}
        self.tel2Entry = {}
        self.FTStatusIcon = {}
        self.refOffsetEntry = {}
        self.FTOffsetEntry = {}
        self.setStateCombo = {}
        hbox = gtk.HBox(False, 0)
        vpaned.add1(hbox)
        for bcndx, bcnb in enumerate(range(1,common.nBL+1)):
            #creates a control panel for each of the BCs
            ############################        
            frame = gtk.Frame("FRINGE %d"%(bcnb))
            frame.set_label_align(0.5,0.5)
            #Create a table that will contain the controls for BC1
            table = gtk.Table(4, 10, False)
            frame.add(table)
            hbox.add(frame)
            tr = 0
            tc = 0
            ############################ 
##             button = gtk.CheckButton("Active fringe")
##             button.connect("toggled", self.setFringeActive, "setActiveButton", bcndx)
##             table.attach(button,tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            label = gtk.Label("TELs")
            label.set_alignment(0,0)
            table.attach(label,tc, tc+1, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            self.tel1Entry[bcndx] = gtk.Entry()
            self.tel1Entry[bcndx].set_text('-')
            self.tel1Entry[bcndx].set_editable(False)
            self.tel1Entry[bcndx].set_size_request(30, 20)
            table.attach(self.tel1Entry[bcndx],tc+1, tc+2, tr, tr+1, gtk.SHRINK,gtk.SHRINK,0,0)
            self.tel2Entry[bcndx] = gtk.Entry()
            self.tel2Entry[bcndx].set_text('-')
            self.tel2Entry[bcndx].set_editable(False)
            self.tel2Entry[bcndx].set_size_request(30, 20)
            #entry.set_tooltip_text("tracking dl")
            #tooltip.connect_object(entry, "tracking dl")
            table.attach(self.tel2Entry[bcndx],tc+2, tc+3, tr, tr+1, gtk.SHRINK,gtk.SHRINK,0,0)
            tr +=1
            #############################
            #sep = gtk.HSeparator()
            #sep.set_size_request(20, 10)
            #table.attach(sep, tc, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            #tr+=1
            #############################
            #label = gtk.Label("DL OFFSET")
            #label.set_alignment(0,0)
            #table.attach(label,tc, tc+1, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            #entry = gtk.Entry()
            #entry.set_editable(False)
            #entry.set_size_request(30, 20)
            #table.attach(entry,tc+1, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            #tr +=1
##             ############################
##             label = gtk.Label("DETECTION THR")
##             label.set_alignment(0,0)
##             table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             entry = gtk.Entry()
##             entry.set_size_request(40, 20)
##             table.attach(entry, tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             tr+=1
##             ############################
##             label = gtk.Label("LOCK THR")
##             label.set_alignment(0,0)
##             table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             entry = gtk.Entry()
##             entry.set_size_request(40, 20)
##             table.attach(entry, tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             tr+=1
##             ############################
##             label = gtk.Label("UNLOCK THR")
##             label.set_alignment(0,0)
##             table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             entry = gtk.Entry()
##             entry.set_size_request(40, 20)
##             table.attach(entry, tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             tr+=1
##             ############################
##             label = gtk.Label("FT GAIN")
##             label.set_alignment(0,0)
##             table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             entry = gtk.Entry()
##             entry.set_size_request(40, 20)
##             table.attach(entry, tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
##             tr+=1
            ############################
            sep = gtk.HSeparator()
            sep.set_size_request(20, 10)
            table.attach(sep, tc, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            tr+=1
            ############################
            label = gtk.Label("FRINGE SEARCH")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            combo_box = gtk.combo_box_new_text()
            searchingModes = ['Paddle','Linear','Spiral']
            for mode in searchingModes:
                combo_box.append_text(mode)
            combo_box.set_active(1)
            combo_box.connect('changed', self.setSearchMode, bcndx)
            table.attach(combo_box,tc+2, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            tr +=1
            ############################
            label = gtk.Label("SPEED [\316\274m/s]")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            self.searchSpeedEntry[bcndx] = gtk.Entry()
            self.searchSpeedEntry[bcndx].set_size_request(30, 20)
            self.searchSpeedEntry[bcndx].set_text("%.1f"%(self.ft.searchSpeed[bcndx]))
            table.attach(self.searchSpeedEntry[bcndx], tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            tr+=1
            ############################
            label = gtk.Label("SPIRAL STEP 0 [mm]")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            self.searchStep0Entry[bcndx] = gtk.Entry()
            self.searchStep0Entry[bcndx].set_size_request(30, 20)
            self.searchStep0Entry[bcndx].set_text("%.1f"%(self.ft.searchStep0[bcndx]))
            table.attach(self.searchStep0Entry[bcndx], tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
##             frame = gtk.Frame("Step \n[mm]")
##             table.attach(frame, tc+3, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)       
##             self.searchStep0Entry[bcndx] = gtk.Entry()
##             self.searchStep0Entry[bcndx].set_size_request(30, 20)
##             self.searchStep0Entry[bcndx].set_text(str(self.fsearch.searchStep0[bcndx]))
##             frame.add(self.searchStep0Entry[bcndx])
            tr+=1
            ############################
            label = gtk.Label("PADDLE STEP [\316\274m]")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            self.paddleStepEntry[bcndx] = gtk.Entry()
            self.paddleStepEntry[bcndx].set_size_request(30, 20)
            self.paddleStepEntry[bcndx].set_text("%.1f"%(self.ft.paddleStep))
            table.attach(self.paddleStepEntry[bcndx], tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            tr+=1
            ############################
            button = gtk.Button('SET PARAMS')
            #button.set_size_request(40, 30)
            button.connect("clicked", self.setSearchParams, bcndx)           
            table.attach(button, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)            
            button = create_arrow_button(gtk.ARROW_LEFT, gtk.SHADOW_ETCHED_IN)
            button.connect("clicked", self.paddleFringe, -1.0,  bcndx)
            button.set_size_request(40, 30)
            table.attach(button, tc+2, tc+3, tr, tr+1, gtk.SHRINK,gtk.SHRINK,0,0)
            button = create_arrow_button(gtk.ARROW_RIGHT, gtk.SHADOW_ETCHED_IN)
            button.set_size_request(40, 30)
            button.connect("clicked", self.paddleFringe, 1.0, bcndx)
            table.attach(button, tc+3, tc+4, tr, tr+1, gtk.SHRINK,gtk.SHRINK,0,0)            
            tr+=1
            ############################
            sep = gtk.HSeparator()
            sep.set_size_request(20, 20)
            table.attach(sep, tc, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            tr+=1
            ############################
            label = gtk.Label("AMP THR")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+1, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            self.ampThrEntry[bcndx] = gtk.Entry()
            self.ampThrEntry[bcndx].set_size_request(40, 20)
            self.ampThrEntry[bcndx].set_text("%.1f"%(self.ft.ampThr[bcndx]))
            table.attach(self.ampThrEntry[bcndx], tc+1, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            tr+=1
            ############################
            button = gtk.Button('SET AMP Thr')
            button.connect("clicked", self.ampThrButton, 'set')
            table.attach(button, tc+0, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)            
            button = gtk.Button('CALC AMP Thr')
            button.connect("clicked", self.ampThrButton, 'calc')
            table.attach(button, tc+2, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)            
            tr+=1
            ############################
            sep = gtk.HSeparator()
            sep.set_size_request(20, 20)
            table.attach(sep, tc, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            tr+=1
            ############################
            label = gtk.Label("REF OFF [mm]")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            self.refOffsetEntry[bcndx] = gtk.Entry()
            self.refOffsetEntry[bcndx].set_size_request(30, 20)
            self.refOffsetEntry[bcndx].set_text("%.3f"%(self.ft.BCRefOffset[bcndx]*1e-3))
            table.attach(self.refOffsetEntry[bcndx], tc+2, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            tr+=1
            ############################
            label = gtk.Label("FT OFF [mm]")
            label.set_alignment(0,0)
            table.attach(label, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            self.FTOffsetEntry[bcndx] = gtk.Entry()
            self.FTOffsetEntry[bcndx].set_size_request(30, 20)
            self.FTOffsetEntry[bcndx].set_text("%.3f"%(self.ft.BCFTOffset[bcndx]*1e-3))
            
            table.attach(self.FTOffsetEntry[bcndx], tc+2, tc+4, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)      
            tr+=1
            #button = gtk.ToggleButton('START')
            #button.set_size_request(40, 30)
            #button.connect("toggled", self.buttonStart, 'lock', bcndx)
            #table.attach(button, tc, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            self.setStateCombo[bcndx] = gtk.combo_box_new_text()
            champStates = ['OFF','SEARCH','SEMI-LOCK','LOCK']
            for state in champStates:
                self.setStateCombo[bcndx].append_text(state)
            self.setStateCombo[bcndx].set_active(0)
            self.setStateCombo[bcndx].connect('changed', self.setChampCommand, bcndx)
            table.attach(self.setStateCombo[bcndx],tc+0, tc+2, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            self.FTStatusIcon[bcndx] = gtk.Image()
            self.FTStatusIcon[bcndx].set_from_file(self.stateIcons[0])
            table.attach(self.FTStatusIcon[bcndx], tc+2, tc+3, tr, tr+1, gtk.FILL,gtk.SHRINK,0,0)
            tr+=1
                   
        #Create a table that will contain the plots
        table = gtk.Table(13, 21, False)
        #bigVBox.add(table)
        vpaned.add2(table)
        
        #Insert animated graphics for all BCs
        table.attach(self.anim.canvas,
            0, 13,                   0, 20,
            gtk.EXPAND|gtk.FILL,               gtk.EXPAND|gtk.FILL,
            0,                      0)
        
        #temporary plots to illustrate possible displays
        import numpy as np
        arr = np.arange(100)
        #self.anim.a[0].plot(arr, np.cos(arr/2.))
        #self.anim.b[0].plot(arr, np.cos(arr/2.)*np.sin(arr/2.))
        #self.anim.c[0].plot(arr, np.cos(arr/2.))
        N = 20
        m = np.zeros((N, 100))
        for i in range(N):
            m[i,:] = np.cos((arr+10*np.sin(i*i/20.))/2.)
        import matplotlib.cm as cm
        #self.anim.d[0].imshow(m, aspect='auto', interpolation=None, cmap = cm.gray, origin = 'lower')
        self.anim.canvas.draw()
        
        toolbar = NavigationToolbar(self.anim.canvas, self)
        table.attach(toolbar, 8, 13, 20,21, gtk.FILL, gtk.SHRINK,0,0)
        
        label = gtk.Label('LIVE DISPLAY')
        table.attach(label, 0, 1, 20,21, gtk.FILL, gtk.SHRINK,0,0)
        button = gtk.ToggleButton('OFF')
        button.set_size_request(40, 26)
        table.attach(button, 1,2, 20,21, gtk.FILL, gtk.SHRINK,0,0)
        button.connect("toggled", self.liveDisplay)
        self.add(bigVBox)
        self.show_all()
    
    def __create_action_group(self):
        # GtkActionEntry
        entries = (
          ( "Cop", None, "_C.O.P" ),               # name, stock id, label
          ( "Set server config", None,                             # name, stock id
            "Set _server config",  "<control>S",                    # label, accelerator
            "Set server config",                                   # tooltip
            self.activate_setServerConfig ),
          ( "Set BC config", None,                             # name, stock id
            "Set _BC config",  "<control>B",                    # label, accelerator
            "Set BC config",                                   # tooltip
            self.activate_setArrayConfig ),
          ( "Set PZT parameters", None,                             # name, stock id
            "Set P_ZT parameters",  "<control>Z",                    # label, accelerator
            "Set PZT parameters",                                   # tooltip
            self.activate_setPZTParameters ),
          ( "Set FT parameters", None,                             # name, stock id
            "Set FT _parameters",  "<control>P",                    # label, accelerator
            "Set FT parameters",                                   # tooltip
            self.activate_setServoParameters ),
          ( "Set/Get offsets", None,                             # name, stock id
            "Set/Get _offsets",  "<control>O",                    # label, accelerator
            "Set/Get offsets",                                   # tooltip
            self.activate_offsetManager ),
          ( "Quit", None,                             # name, stock id
            "_Quit", "<control>Q",                    # label, accelerator
            "Quit",                                   # tooltip
            self.activate_quit ),
          ( "HelpMenu", None, "_Help" ),               # name, stock id, label
          ( "Help", None,                             # name, stock id
            "_Help",  "<control>H",                    # label, accelerator
            "Help",                                   # tooltip
            self.activate_help ),
        );

        # Create the menubar and toolbar
        action_group = gtk.ActionGroup("AppWindowActions")
        action_group.add_actions(entries)
        return action_group
    def activate_setServerConfig(self, action):
        champControls.setServerConfig(parent=self)
        
    def activate_setArrayConfig(self, action):
        champControls.setArrayConfig(parent=self)

    def activate_setPZTParameters(self, action):
        champControls.setPZTParameters(parent=self)

    def activate_setServoParameters(self, action):
        champControls.setServoParameters(parent=self)

    def activate_offsetManager(self, action):
        champControls.offsetManager(parent=self)

    def activate_quit(self, action):
        print 'Quitting program'
        gtk.main_quit()

    def activate_help(self, action):
        champHelp.HypertextHelp(parent=self)
        
    def paddleFringe(self, widget, sign=None, bc=None):
        #manually moves fringes
        self.ft.paddleOffset = sign
        self.ft.paddleFringe(bc,sign*float(self.paddleStepEntry[bc].get_text()))
        
    def setSearchMode(self, widget, bc=None):
        #set the search fringe mode
        status = widget.get_active()
        self.ft.searchMode[bc]=status
        #print status, self.fsearch.searchMode

    def setSearchParams(self, widget, bc=None):
        #set the search fringe mode
        self.ft.searchSpeed[bc] = float(self.searchSpeedEntry[bc].get_text())
        self.ft.searchStep0[bc] = float(self.searchStep0Entry[bc].get_text())
        self.ft.setSearchParameters()
        
    def setChampCommand(self, widget, bc=None):
        timeout = 1000     #milliseconds
        status = widget.get_active()
        self.FTStatusIcon[bc].set_from_file(self.stateIcons[status])
        self.ft.champSetState = np.array([999,999,999,999,999,999])
        self.ft.champSetState[bc]=status
        self.ft.champCommand()
        self.ft.getChampStatus()
        nonzero = len(np.nonzero(self.ft.champState)[0])
        print "nonzero = ", nonzero
        if status in [1,2,3]:
            if self.tagFTLoop == -1:
                print "start loop"
                self.tagFTLoop = gobject.timeout_add(timeout, self.FTLoop)
        elif nonzero == 0:
            if self.tagFTLoop != -1:
                print "stop loop"
                gobject.source_remove(self.tagFTLoop)
                self.tagFTLoop = -1
        
    def buttonStart(self, widget, data=None, bc=None):
        status = widget.get_active()
        if status:
            widget.set_label('STOP')
            self.ft.champCommand()
        else:
            widget.set_label('START')
            self.ft.champCommand()
       
    def ampThrButton(self, widget, data=None):
        if data == 'set':
            for i in range(common.nBL):
                self.ft.ampThr[i] = float(self.ampThrEntry[i].get_text())
            self.ft.setAmpThr()
        elif data == 'calc':
            self.ft.calcAmpThr()
            self.ampThrEntry[i].set_text()
        
    def buttonLock(self, widget, data=None, bc=None):
        if widget.get_active() : widget.set_label('LOCK')
        else: widget.set_label('LOCK')
        
    def buttonAuto(self, widget, data=None, bc=None):
        if widget.get_active() : widget.set_label('AUTO ON')
        else: widget.set_label('AUTO OFF')

    def FTLoop(self):
        print "In loop"
        self.ft.getChampStatus()
        gtk.gdk.threads_enter()
        try:
            for i in range(common.nBL):
                self.FTStatusIcon[i].set_from_file(self.stateIcons[self.ft.champState[i]])
                self.setStateCombo[i].set_active(self.ft.champState[i])
                self.FTOffsetEntry[i].set_text(str(self.ft.BCRefOffset[i]*1e-3))
                self.refOffsetEntry[i].set_text(str(self.ft.BCFTOffset[i]*1e-3))

                if common.liveDisplayStatus:
                    print "HERE1"
                    #self.ft.getFringeData()
                    #common.dataFluxReady = 1
                    print "HERE2"
                    self.anim.updateFig()
                    print "HERE3"
        finally:
            gtk.gdk.threads_leave()
        return True
            
    def liveDisplay(self, widget):
        if widget.get_active() : 
            widget.set_label('ON')
            common.liveDisplayStatus = 1
        else: 
            widget.set_label('OFF')
            common.liveDisplayStatus = 0

def main():
    ApplicationMainWindow()
    gtk.gdk.threads_enter()
    gtk.main()
    gtk.gdk.threads_leave()

if __name__ == '__main__':
    main()
