#!/usr/bin/env python
    
# example base.py
import pygtk
pygtk.require('2.0')
import mounts
import gtk
import arrows
import picoData
import picoLowLevel
import numarray as PicoLib  

	
class Base(picoLowLevel.PicoControl):
		
	
	def __init__(self):
		
		# Start GUI Assembly
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.button = gtk.Button()
		self.table = gtk.Table()
		
		self.window.connect("delete_event", self.delete_event)
		self.window.connect("destroy", self.destroy)	
		self.window.set_border_width(10)
		
		table = gtk.Table(7, 5, False)
        	self.window.add(table)
		self.window.show()

		#Overhead Menu
		file_menu = gtk.Menu()
		quit = gtk.MenuItem("Quit")
		mode = gtk.MenuItem("Mode")
		joymap = gtk.MenuItem("Joystick Mapping")
		control = gtk.MenuItem("Control")
		mode_menu = gtk.Menu()
		mode_menu.append(joymap)
		mode_menu.append(control)
		
		mode.set_submenu(mode_menu)		

		file_menu.append(mode)
		file_menu.append(quit)

		# Menu Bar Callbacks
		quit.connect("activate",self.destroy)
		joymap.connect("activate", self.mode_change, 'joy')
		control.connect("activate", self.mode_change, 'cont')


		quit.show()
		menu_bar = gtk.MenuBar()
		table.attach(menu_bar, 0,8,0,1, gtk.FILL, gtk.FILL,0,0);
		menu_bar.show()
		top_menu = gtk.MenuItem("Menu")
		top_menu.show()
		top_menu.set_submenu(file_menu)
		menu_bar.append(top_menu)
		
		
		self.mount_buttons, x_pos, y_pos = self.create_mount_buttons()
		
		# Button for F-T.6
		table.attach(self.mount_buttons[0], 0, 1, 1, 2, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[0].connect("clicked", self.mounts_callback,0)


		# Button for F-T.1
		table.attach(self.mount_buttons[1], 0, 1, 2, 3, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[1].connect("clicked", self.mounts_callback,1)


		# Button for F-T.2
		table.attach(self.mount_buttons[2], 0, 1, 3, 4, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[2].connect("clicked", self.mounts_callback,2)


		# Button for F-T.3
		table.attach(self.mount_buttons[3], 0, 1, 4, 5, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[3].connect("clicked", self.mounts_callback,3)


		# Button for F-T.4
		table.attach(self.mount_buttons[4], 0, 1, 5, 6, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[4].connect("clicked", self.mounts_callback,4)


		# Button for F-T.5
		table.attach(self.mount_buttons[5], 0, 1, 6, 7, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[5].connect("clicked", self.mounts_callback,5)


		# Button for BC.6,1
		table.attach(self.mount_buttons[6], 1, 2, 1, 2, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[6].connect("clicked", self.mounts_callback,6)


		# Button for BC.1,2
		table.attach(self.mount_buttons[7], 1, 2, 2, 3, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[7].connect("clicked", self.mounts_callback,7)
	

		# Button for BC.2,3
		table.attach(self.mount_buttons[8], 1, 2, 3, 4, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[8].connect("clicked", self.mounts_callback,8)
	

		# Button for BC.3,4
		table.attach(self.mount_buttons[9], 1, 2, 4, 5, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[9].connect("clicked", self.mounts_callback,9)


		# Button for BC.4,5
		table.attach(self.mount_buttons[10], 1, 2, 5, 6, gtk.FILL,  gtk.FILL, 0, 0);
		self.mount_buttons[10].connect("clicked", self.mounts_callback,10)
		
		# Button for BC.5,6
		table.attach(self.mount_buttons[11], 1, 2, 6, 7, gtk.FILL | gtk.EXPAND,  gtk.FILL, 0, 0);
		self.mount_buttons[11].connect("clicked", self.mounts_callback,11)


		# Adds separator between button field and control panel
		separator = gtk.VSeparator()
        	separator.set_size_request(20, 300)
        	table.attach(separator, 2, 3, 1, 6, gtk.FILL, 0, 0, 0);
		self.window.show_all()	
		# Create Arrow buttons
		self.arrow_buttons = self.create_arrow_buttons()

		

		# Up Arrow of control panel
		#arrow_buttons, x_pos, y_pos = self.create_arrow_buttons('up')
		table.attach(self.arrow_buttons[2],6,7,2,3,0, 0, 0, 0);
		self.arrow_buttons[2].connect("clicked", self.push_to_move_callback,'up')

		# Down Arrow of control panel
		table.attach(self.arrow_buttons[3],6,7,4,5,0, 0, 0, 0);
		self.arrow_buttons[3].connect("clicked", self.push_to_move_callback,'down')
		
		
		# Right Arrow of control panel
		table.attach(self.arrow_buttons[1],7,8,3,4,0, 0, 0, 0);		
		self.arrow_buttons[1].connect("clicked", self.push_to_move_callback,'right')

		# Left Arrow of control panel
		table.attach(self.arrow_buttons[0],5,6,3,4,0, 0, 0, 0);
		self.arrow_buttons[0].connect("clicked", self.push_to_move_callback,'left')
		
		self.entry = []
		self.entrySteps = []

		# Label for selected Picomotor
		self.entry.append(gtk.Entry())
		self.entry[0].set_size_request(80,20)
		self.entry[0].set_text(picoData.label)
		self.entry[0].set_editable(False)		
		self.frame0 = gtk.Frame("Active Picomotor")
		self.frame0.add(self.entry[0])
		table.attach(self.frame0,5,7,1,2,gtk.FILL, 0,0,0);
		
		# Step Size on X Axis
		self.entry.append(gtk.Entry())
		self.entry[1].set_size_request(60,20)
		self.entry[1].set_text("not done")
		self.entry[1].set_editable(True)
		self.frame1 = gtk.Frame("X Axis Step Size")
		self.frame1.add(self.entry[1])
		table.attach(self.frame1,6,7,5,6,gtk.FILL,0,0,0);
		self.entry[1].connect("activate",self.update_step_callback, 'X')
		
		# Step Size on Y Axis
		self.entry.append(gtk.Entry())
		self.entry[2].set_size_request(60,20)
		self.entry[2].set_text("not done")
		self.entry[2].set_editable(True)
		self.frame2 = gtk.Frame("Y Axis Step Size")
		self.frame2.add(self.entry[2])
		table.attach(self.frame2,6,7,6,7,gtk.FILL,0,0,0);
		self.entry[2].connect("activate",self.update_step_callback, 'Y')
		
		
		

	# Moves picomotor at push of arrow button.
	def push_to_move_callback(self, widget, axis):
		print axis
		if axis == 'up':
			self.MoveRelative(self.driver[1],self.motor[1],-self.steps[1])
		if axis == 'down':
			self.MoveRelative(self.driver[1],self.motor[1],self.steps[1])
		if axis == 'left':
			self.MoveRelative(self.driver[0],self.motor[0],self.steps[0])
		if axis == 'right':
			self.MoveRelative(self.driver[0],self.motor[0], -self.steps[0])
		


	# Sets label on picomotor selection and Loads control of selected picomotor.
	def mounts_callback(self, widget, num):
		self.window.show_all()
		picoData.label = picoData.picoId.field('picoId')[num*2]
		self.entry[0].set_text(picoData.label)
		picoName = picoData.label
		picoIP = picoData.getFieldValue(picoData.picoId, 'IP', 'picoId', picoName)
        	index, = picoData.picoId.field('picoId').match(picoName)
        	nAxis = len(index)
		self.driver = picoData.picoId.field('driver')[index]
		self.motor = picoData.picoId.field('motor')[index]
		self.steps = PicoLib.array([500]*nAxis)
		self.Relative = PicoLib.zeros(nAxis)
		print "picoName = ", picoName
        	print "available axis: ", picoData.picoId.field('axis')[index]
		print "picoIP = ", picoIP
		print self.motor
 		
		# Intialization of the telnet connection with the picoMotor
        	if picoData.select_Flag == False:
			picoLowLevel.PicoControl.__init__(self, picoIP)	
      			print "self.tn.host = ", self.tn.host
		for i in range(nAxis):
	        	self.SetVelocity(self.driver[i], self.motor[i], 2000)
		self.entry[1].set_text(str(self.steps[0]))
		self.entry[2].set_text(str(self.steps[1]))
		picoData.select_Flag = True

	# End program on close
	def delete_event(self, widget, data=None):
		print "delete event occurred"
		self.destroy
		return False
	def destroy(self, widget, data=None):
		#picoLowLevel.PicoControl.__del__(self)
		gtk.main_quit
		self.destroy
		print "check"

	# Create icons for mounts to place on buttons
	def create_mount_buttons(self):
		mount_buttons = []
		x_pos = [0, 0, 0, 0]
		y_pos = [0, 0, 0, 0]
		for n in range(12):	
			mount_buttons.append(gtk.Button())
			if n < 6:
				image = mounts.makeIcon('right')
			else:
				image = mounts.makeIcon('left')
			mount_buttons[n].add(image)
		x_pos = [0, 0, 0, 0]
		y_pos = [0, 0, 0, 0]
		return mount_buttons, x_pos, y_pos

	# Create icons for arrows to place on buttons
	def create_arrow_buttons(self):
		arrow_buttons = []
		x_pos = [0, 0, 0, 0]
		y_pos = [0, 0, 0, 0]
		
		# Makes left arrow button
		arrow_buttons.append(gtk.Button())
		image = arrows.makeIcon('left')
		arrow_buttons[0].add(image)

		# Makes right arrow button
		arrow_buttons.append(gtk.Button())
		image = arrows.makeIcon('right')
		arrow_buttons[1].add(image)

		# Makes up arrow button
		arrow_buttons.append(gtk.Button())
		image = arrows.makeIcon('up')
		arrow_buttons[2].add(image)

		# Makes down arrow button
		arrow_buttons.append(gtk.Button())
		image = arrows.makeIcon('down')
		arrow_buttons[3].add(image)

		x_pos = [0, 0, 0, 0]
		y_pos = [0, 0, 0, 0]
		return arrow_buttons
	
	# Updates step size when enter is pressed in the text box	
	def update_step_callback(self, widget, axis):
		if axis == 'X':
			pointer = 1
		if axis == 'Y':
			pointer =2
		newsteps = self.entry[pointer].get_text()
		self.steps[pointer-1] = int(newsteps)
		print "New step size: %d\n" % self.steps[pointer-1]
   		
	def mode_change(self, widget, mode):
		if mode == 'cont':
			print "nothing"
		if mode == 'joy':
			self.arrow_buttons[0].hide()
			self.arrow_buttons[0].unmap()
			self.arrow_buttons[1].hide()
			self.arrow_buttons[1].unmap()
			self.arrow_buttons[2].hide()
			self.arrow_buttons[2].unmap()
			self.arrow_buttons[3].hide()
			self.arrow_buttons[3].unmap()
			self.entry[0].hide()
			self.entry[0].unmap()
			self.entry[1].hide()
			self.entry[1].unmap()
			self.entry[2].hide()
			self.entry[2].unmap()
			self.frame0.hide()
			self.frame0.unmap()
			self.frame1.hide()
			self.frame1.unmap()
			self.frame2.hide()
			self.frame2.unmap()
			
	def main(self):
		gtk.main()		

if __name__ == "__main__":
	base=Base()	
	base.main()
